// ignore_for_file: import_of_legacy_library_into_null_safe, unnecessary_null_comparison

import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:date_format/date_format.dart';
import 'package:device_info/device_info.dart';
import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter_xi_platform/src/xi_constants.dart';
import 'package:flutter_xi_platform/src/xi_device_info_model.dart';
import 'package:package_info/package_info.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:encrypt/encrypt.dart' as encrypt;
import 'package:connectivity/connectivity.dart';

import 'xi_event_model.dart';

const kFirstLaunchEventDefaultsKey = "kFirstLaunchEventDefaultsKey";
const kFirstLaunchTimeDefaultsKey = "kFirstLaunchTimeDefaultsKey";
const kEventsMonetizationDefaultsKey = "kEventsMonetizationDefaultsKey";
const kEventsUserBehaviorDefaultsKey = "kEventsUserBehaviorDefaultsKey";
const kRequestBodyDataKey = "data";
const kEventPostDataAESSecretKey = "9g8ndXVCDLxUc3rF";

class XiEventSubmiter {
  static final XiEventSubmiter _singleton = XiEventSubmiter._internal();

  factory XiEventSubmiter() {
    return _singleton;
  }

  XiEventSubmiter._internal();

  bool _paramsIsOK = false;
  late Dio _dio;
  bool _debugLogEnable = false;
  var _deviceInfo = XiEventDeviceInfoModel();

  XiEventSubmiter configDebugEnable(bool debugEnable) {
    _debugLogEnable = debugEnable;
    return this;
  }

  XiEventSubmiter configCustomerUserId(String customerUserId) {
    _deviceInfo.customerUserId = customerUserId;
    return this;
  }

  XiEventSubmiter configPlatformName(String platformName) {
    _deviceInfo.appName = platformName;
    return this;
  }

  XiEventSubmiter configGoogleId(String googleId) {
    _deviceInfo.googleId = googleId;
    return this;
  }

  XiEventSubmiter configCountryCode(String countryCode) {
    _deviceInfo.countryCode = countryCode;
    return this;
  }

  XiEventSubmiter configIDfA(String idfa) {
    _deviceInfo.idfa = idfa;
    return this;
  }

  Future<void> trackFirstLaunch() async {
    var prefs = await SharedPreferences.getInstance();
    var has = prefs.getBool(kFirstLaunchEventDefaultsKey);
    if (has == null) {
      XiEventModel eventModel = XiEventModel();
      eventModel.deviceInfo = _deviceInfo;
      eventModel.eventName = XiEventFirstLaunch;
      eventModel.firstLaunchTime = await _getFirstLaunchTimeString;
      eventModel.time = eventModel.firstLaunchTime;

      bool success = await _addEvent(eventModel);
      if (success) {
        await prefs.setBool(kFirstLaunchEventDefaultsKey, true);
      }
    }
  }

  Future<void> trackLoginEvent() async {
    XiEventModel eventModel = XiEventModel();

    eventModel.deviceInfo = _deviceInfo;
    eventModel.eventName = XiEventLaunch;
    eventModel.firstLaunchTime = await _getFirstLaunchTimeString;
    eventModel.time = _currentTimeString;
    _addEvent(eventModel);
  }

  trackAdEvent(
    String eventName,
    String platform,
    String placementKey,
    String placementId,
  ) async {
    XiEventModel eventModel = XiEventModel();
    eventModel.deviceInfo = _deviceInfo;
    eventModel.eventName = eventName;
    eventModel.adPlatform = platform;
    eventModel.placementKey = placementKey;
    eventModel.placementId = placementId;
    eventModel.firstLaunchTime = await _getFirstLaunchTimeString;
    eventModel.time = _currentTimeString;

    _addEvent(eventModel);
  }

  trackIAPEvent(
    String eventName,
    String productId,
    double price,
    String currencyCode, [
    String? receipt,
    String? applePassword,
  ]) async {
    XiEventModel eventModel = XiEventModel();
    eventModel.deviceInfo = _deviceInfo;
    eventModel.eventName = eventName;
    eventModel.productId = productId;
    eventModel.price = price;
    eventModel.currencyCode = currencyCode;
    eventModel.receipt = receipt;
    eventModel.applePassword = applePassword;
    eventModel.firstLaunchTime = await _getFirstLaunchTimeString;
    eventModel.time = _currentTimeString;
    if (XiEventSubscription == eventName) {
      eventModel.service = "apple";
      eventModel.receipt = receipt;
      eventModel.applePassword = applePassword;
    }
    _addEvent(eventModel);
  }

  trackBehaviorEvent(
    String eventName,
    Map<String, String> customInfo,
  ) async {
    XiEventModel eventModel = XiEventModel();

    eventModel.deviceInfo = _deviceInfo;
    eventModel.eventName = eventName;
    eventModel.customInfo = customInfo;
    eventModel.firstLaunchTime = await _getFirstLaunchTimeString;
    eventModel.time = eventModel.firstLaunchTime;
    _addEvent(eventModel);
  }

  Future<bool> _addEvent(XiEventModel event) async {
    if (!await _setupEssentials()) {
      return false;
    }

    String submitPath;
    switch (event.eventType) {
      case XiEventType.monetization:
        submitPath = XiEventMonetizationServerPath;
        break;
      case XiEventType.userBehavior:
        submitPath = XiEventUserBehaviorServerPath;
        break;
      default:
        return false;
    }

    var list = <Map<String, dynamic>>[];
    var dict = event.toMap();

    String firstLaunchTime = dict["first_launch_time"];
    if (firstLaunchTime != null && firstLaunchTime.length > 0) {
      dict["init_time"] = firstLaunchTime;
      dict.remove("first_launch_time");
    }

    Map deviceInfo = dict["device_info"];
    if (deviceInfo != null && deviceInfo.length > 0) {
      deviceInfo.forEach((key, value) {
        dict[key] = value;
      });
      dict.remove("device_info");
    }

    Map customInfo = dict["custom_info"];
    if (customInfo != null && customInfo.length > 0) {
      customInfo.forEach((key, value) {
        dict[key] = value;
      });
      dict.remove("custom_info");
    }

    var filtedDict = Map<String, dynamic>();
    dict.forEach((key, value) {
      if (value != null) {
        filtedDict[key] = value;
      }
    });
    list.add(filtedDict);

    _printLog("Submit events:$list");

    final key = encrypt.Key.fromUtf8(kEventPostDataAESSecretKey);
    final encrypter =
        encrypt.Encrypter(encrypt.AES(key, mode: encrypt.AESMode.ecb));
    var encryptedString = encrypter
        .encrypt(jsonEncode(list), iv: encrypt.IV.fromLength(16))
        .base64;
    var domainPath =
        _debugLogEnable ? XiEventTestServerAddress : XiEventOnlineServerAddress;
    var urlString = domainPath + submitPath;

    try {
      Response response =
          await dioClient.post(urlString, data: {"data": encryptedString});
      if (response.data != null &&
          response.data["code"] != null &&
          response.data["code"] == 200) {
        _printLog("Submit events success:$list");
        return true;
      }
      throw ("response error");
    } catch (e) {
      _printLog("Submit events error:$list===>${e.toString()}");
      return false;
    }
  }

  Future<bool> _setupEssentials() async {
    if (_paramsIsOK) {
      return true;
    }

    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    var connectivityResult = await (Connectivity().checkConnectivity());

    if (Platform.isIOS) {
      var deviceInfo = await DeviceInfoPlugin().iosInfo;

      _deviceInfo.platform = "ios";
      _deviceInfo.idfv = deviceInfo.identifierForVendor;
      _deviceInfo.osVersion = deviceInfo.systemVersion;
    } else {
      var deviceInfo = await DeviceInfoPlugin().androidInfo;

      _deviceInfo.platform = "android";
      _deviceInfo.osVersion = deviceInfo.version.release;
      _deviceInfo.androidId = deviceInfo.androidId;
    }
    _deviceInfo.bundleId = packageInfo.packageName;
    _deviceInfo.appVersion = packageInfo.version;
    _deviceInfo.wifi = connectivityResult == ConnectivityResult.wifi;

    if (_deviceInfo.customerUserId == null) {
      throw "missing essential param: [customerUserId]";
    }
    if (_deviceInfo.platform == null) {
      throw "missing essential param: [platform]";
    }
    if (_deviceInfo.countryCode == null) {
      throw "missing essential param: [countryCode]";
    }
    if (Platform.isIOS) {
      if (_deviceInfo.idfa == null) {
        throw "missing essential param: [idfa]";
      }
    } else {
      if (_deviceInfo.googleId == null) {
        throw "missing essential param: [googleId]";
      }
    }

    _paramsIsOK = true;
    return Future.value(true);
  }

  Future<String?> get _getFirstLaunchTimeString async {
    var prefs = await SharedPreferences.getInstance();
    String? timeString = prefs.getString(kFirstLaunchTimeDefaultsKey);
    if (timeString == null || timeString.length == 0) {
      timeString = _currentTimeString;
    }
    await prefs.setString(kFirstLaunchTimeDefaultsKey, timeString);
    return timeString;
  }

  String get _currentTimeString {
    var time = DateTime.now().toUtc();
    var string = formatDate(time, [
      "yyyy",
      "-",
      "mm",
      "-",
      "dd",
      "T",
      "HH",
      ":",
      "mm",
      ":",
      "ss",
      ".",
      "SSS",
    ]);
    return string + "Z";
  }

  Dio get dioClient {
    if (_dio == null) {
      var dio = Dio();
      // (dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
      //     (client) {
      //   client.findProxy = (uri) {
      //     return 'PROXY 192.168.8.17:8888';
      //   };
      // };

      dio.options.connectTimeout = 15000;
      dio.options.receiveTimeout = 15000;
      dio.options.responseType = ResponseType.json;
      dio.options.headers = {
        "Content-Type": "application/json",
      };
      dio.options.validateStatus = (int? status) {
        return true;
      };
      _dio = dio;
    }
    return _dio;
  }
  void _printLog(String content) {
    if (_debugLogEnable == true) {
      debugPrint(content);
    }
  }
}
